package ohnosequences.mg7

import ohnosequences.datasets._
import ohnosequences.cosas._, types._, klists._

case object data {

  // Flash:
  case object pairedReads1 extends FileData("reads1")("fastq.gz")
  case object pairedReads2 extends FileData("reads2")("fastq.gz")

  case object mergedReads extends FileData("reads")("fastq")
  case object flashStats extends FileData("stats")("txt")

  case object flashInput extends DataSet(pairedReads1 :×: pairedReads2 :×: |[AnyData])
  case object flashOutput extends DataSet(mergedReads :×: flashStats :×: |[AnyData])


  // Reads after splitting (multiple files in a virtual S3 folder):
  case object fastaChunks extends Data("reads-chunks")

  case object splitInput extends DataSet(mergedReads :×: |[AnyData])
  case object splitOutput extends DataSet(fastaChunks :×: |[AnyData])


  // Blast input:
  case object fastaChunk extends FileData("reads")("fastq")
  // Blast output for each chunk:
  case object blastChunkOut extends FileData("blast.chunk")("csv")

  case object blastInput extends DataSet(fastaChunk :×: |[AnyData])
  case object blastOutput extends DataSet(blastChunkOut :×: |[AnyData])


  // all output chunks together:
  case object blastChunks extends Data("blast-chunks")
  // after merging chunks:
  case object blastResult extends FileData("blast")("csv")

  case object mergeInput extends DataSet(blastChunks :×: |[AnyData])
  case object mergeOutput extends DataSet(blastResult :×: |[AnyData])


  // Assignment output:
  case object lcaCSV extends FileData("lca")("csv")
  case object bbhCSV extends FileData("bbh")("csv")

  case object assignmentInput extends DataSet(blastResult :×: |[AnyData])
  case object assignmentOutput extends DataSet(lcaCSV :×: bbhCSV :×: |[AnyData])


  // Counting output:
  case object lcaDirectCountsCSV extends FileData("lca.direct.counts")("csv")
  case object bbhDirectCountsCSV extends FileData("bbh.direct.counts")("csv")
  case object lcaAccumCountsCSV extends FileData("lca.accum.counts")("csv")
  case object bbhAccumCountsCSV extends FileData("bbh.accum.counts")("csv")

  case object countingInput extends DataSet(lcaCSV :×: bbhCSV :×: |[AnyData])
  case object countingOutput extends DataSet(
    lcaDirectCountsCSV :×:
    bbhDirectCountsCSV :×:
    lcaAccumCountsCSV :×:
    bbhAccumCountsCSV :×:
    |[AnyData]
  )

}
